/* Copyright R. Gentleman, 2003, all rights reserved

 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Ruuid.h"

#ifdef WIN32
uuid_t *UUID2uuidt(UUID curUUID) {
    struct uuid tmpUuid;
    uuid_t *newuuid;

    tmpUuid.time_low = curUUID.Data1;
    tmpUuid.time_mid = curUUID.Data2;
    tmpUuid.time_hi_and_version = curUUID.Data3;
    /* Copy the first two bytes of Data4 into clock_seq */
    memcpy(&tmpUuid.clock_seq, curUUID.Data4, 2);
    /* And the last 6 bytes into node */
    memcpy(tmpUuid.node, curUUID.Data4+2, 6);
    /* Now have a complte tsouid structure, convert to Ts'o uuid_t */
    newuuid = (uuid_t *)R_alloc(1,sizeof(uuid_t));
    uuid_pack(&tmpUuid, *newuuid);

    return(newuuid);
}
#endif

uuid_t *getuuid(void) {
    uuid_t *newuuid;

    newuuid = (uuid_t *)R_alloc(1,sizeof(uuid_t));
#ifdef WIN32
    UUID newWinUUID;
    /* Create the Win32 UUID */
    if (UuidCreate(&newWinUUID) == RPC_S_UUID_NO_ADDRESS) {
	/* Error, can not generate the UUID */
	return(NULL);
    }
    newuuid = UUID2uuidt(newWinUUID);
#else
    uuid_generate(*newuuid);
#endif
    return(newuuid);
}

SEXP Ruuid_getuuid(void)
{
    uuid_t *newuuid;
    char x[UUIDSTRLEN];
    SEXP RuClass, curRuuid, uClass, curUuidt, z;

    RuClass = MAKE_CLASS("Ruuid");
    uClass = MAKE_CLASS("uuidt");
    PROTECT(curRuuid = NEW_OBJECT(RuClass));
    newuuid = getuuid();
    if (newuuid == NULL)
	return(R_NilValue);
    strncpy(x,uuid2str(newuuid),UUIDSTRLEN);
    SET_SLOT(curRuuid, Rf_install("stringRep"), mkString(x));
    PROTECT(curUuidt = NEW_OBJECT(uClass));
    PROTECT(z = allocVector(RAWSXP, UUIDTLEN));
    memcpy(RAW(z), *newuuid, sizeof(uuid_t));
    SET_SLOT(curUuidt, Rf_install("uuid"), z);
    SET_SLOT(curRuuid, Rf_install("uuid"), curUuidt);
    UNPROTECT(3);
    return(curRuuid);
}

char *uuid2str(uuid_t *x) {
    char *y;

    y = (char *)R_alloc(UUIDSTRLEN,sizeof(char));

    /* Perform the conversion */
    uuid_unparse(*x, y);

    /* Need to manuallyadd in the NULL */
    y[UUIDSTRLEN] = '\0';

    return(y);
}

uuid_t *str2uuid(const char *x) {
    uuid_t *y;
    int err;

    /* Check that the string is UUIDSTRLEN-1 (subtraction for the NULL
       character) */
    if (strlen(x) != (UUIDSTRLEN-1))
	error("Invalid string parameter");

    y = (uuid_t *)R_alloc(1,sizeof(uuid_t));

    /* Convert */
    /* uuid_parse doesn't modify, the case gets rid of a warning */
    err = uuid_parse((char *)x, *y);
    if(err)
	error("Error transforming: %d \n", err);

    return(y);
}

SEXP Ruuid_uuid2str(SEXP x)
{
  uuid_t myuuid;

  memcpy(myuuid, RAW(GET_SLOT(x,Rf_install("uuid"))), sizeof(uuid_t));

  return(mkString(uuid2str(&myuuid)));
}


SEXP Ruuid_str2uuid(SEXP x)
{
    SEXP uClass, curUuid, rawid;
    uuid_t *y;

    if(!isString(x))
	error("require a string argument");

    y = str2uuid(STR(x));

    uClass = MAKE_CLASS("uuidt");
    PROTECT(curUuid = NEW_OBJECT(uClass));

    PROTECT(rawid = allocVector(RAWSXP, UUIDTLEN));
    memcpy(RAW(rawid), *y, sizeof(uuid_t));

    SET_SLOT(curUuid, Rf_install("uuid"), rawid);
    UNPROTECT(2);

  return(curUuid);
}

SEXP Ruuid_printuuid(SEXP uuid)
{
    char outStr[100];
    uuid_t uu;
    int i;

    strcpy(outStr,"\0");

    memcpy(uu, RAW(GET_SLOT(uuid,Rf_install("uuid"))), sizeof(uuid_t));

    for(i=0; i<16; i++)
	sprintf(outStr,"%s:%d:",outStr,uu[i]);

    return(mkString(outStr));
}


